defmodule DecentralisedBookIndex.Sync.BookSync do
  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.Metadata.Book
  alias DecentralisedBookIndex.Sync.BookEditionsRegistrySync

  def create_update(attrs, server_id) do
    registry_info = sync_registry(attrs, server_id)

    case Metadata.get_book_by_id(attrs.id) do
      {:ok, book} ->
        attrs =
          attrs
          |> Map.delete(:id)
          |> update_registry(registry_info)
          |> Map.put(:dbi_server_id, server_id)

        book
        |> Ash.Changeset.for_update(:sync, attrs)
        |> Ash.update!(authorize?: false)

        :ok
      {:error, %Ash.Error.Query.NotFound{}} ->
        attrs =
          attrs
          |> update_registry(registry_info)
          |> Map.put(:dbi_server_id, server_id)

        Book
        |> Ash.Changeset.for_create(:sync_create, attrs)
        |> Ash.create!(authorize?: false)

        :ok
    end
  end

  defp sync_registry(attrs, server_id) do
    id = Map.get(attrs, :book_editions_registry_id, nil)

    if id != nil do
      %{id: id}
      |> BookEditionsRegistrySync.create_update(server_id)

      {:ok, id}
    else
      {:error, :no_book_editions_registry_id}
    end
  end

  defp update_registry(attrs, registry_info) do
    case registry_info do
      {:ok, id} ->
        attrs
        |> Map.replace(:book_editions_registry_id, id)

      {:error, _} ->
        attrs
        |> Map.delete(:book_editions_registry_id)
    end
  end
end
