defmodule DecentralisedBookIndexWeb.AuthorLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="author-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:name]} type="text" label="Name" />
          <.input field={@form[:avatar_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.input
            field={@form[:author_alias_registry_id]}
            type="text"
            label="Author alias registry"
          />
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:name]} type="text" label="Name" />
          <.input field={@form[:avatar_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.input
            field={@form[:author_alias_registry_id]}
            type="text"
            label="Author alias registry"
          />
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"author" => author_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, author_params))}
  end

  def handle_event("save", %{"author" => author_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: author_params) do
      {:ok, author} ->
        notify_parent({:saved, author})

        socket =
          socket
          |> put_flash(:info, "Author #{socket.assigns.form.source.type}d successfully")
          |> push_redirect(to: patch_url(socket.assigns.action, author.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    author = socket.assigns.author
    author_id = if author == nil, do: nil, else: author.id

    socket =
      socket
      |> push_redirect(to: patch_url(socket.assigns.action, author_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{author: author}} = socket) do
    form =
      if author do
        AshPhoenix.Form.for_update(author, :update,
          as: "author",
          actor: socket.assigns.current_user
        )
      else
        AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.Author, :create,
          as: "author",
          actor: socket.assigns.current_user
        )
      end

    assign(socket, form: to_form(form))
  end

  defp patch_url(action, author_id) do
    case action do
      :edit -> ~p"/authors/#{author_id}"
      :new  -> ~p"/authors"
       _    -> ~p"/authors"
    end
  end
end
