defmodule DecentralisedBookIndexWeb.PublisherLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="publisher-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:name]} type="text" label="Name" />
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:name]} type="text" label="Name" />
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"publisher" => publisher_params}, socket) do
    {:noreply,
     assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, publisher_params))}
  end

  def handle_event("save", %{"publisher" => publisher_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: publisher_params) do
      {:ok, publisher} ->
        notify_parent({:saved, publisher})

        socket =
          socket
          |> put_flash(:info, "Publisher #{socket.assigns.form.source.type}d successfully")
          |> push_redirect(to: patch_url(socket.assigns.action, publisher.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    publisher = socket.assigns.publisher
    publisher_id = if publisher == nil, do: nil, else: publisher.id

    socket =
      socket
      |> push_redirect(to: patch_url(socket.assigns.action, publisher_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{publisher: publisher}} = socket) do
    form =
      if publisher do
        AshPhoenix.Form.for_update(publisher, :update,
          as: "publisher",
          actor: socket.assigns.current_user
        )
      else
        AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.Publisher, :create,
          as: "publisher",
          actor: socket.assigns.current_user
        )
      end

    assign(socket, form: to_form(form))
  end

  defp patch_url(action, publisher_id) do
    case action do
      :edit -> ~p"/publishers/#{publisher_id}"
      :new  -> ~p"/publishers"
       _    -> ~p"/publishers"
    end
  end
end
