defmodule DecentralisedBookIndex.Metadata.Forms.AuthorFormTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Metadata

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "form submiting" do
    test "user can't submit" do
      user = generate(user())

      valid_params = %{
        "name" => "Oleh",
        "description" => "A cool author",
      }


      assert_raise Ash.Error.Forbidden, fn ->
      form = \
        AshPhoenix.Form.for_create(Metadata.Author, :create,
          as: "author",
          actor: user
        )
          |> AshPhoenix.Form.ensure_can_submit!()
      end
    end

    test "moderator can submit", %{user: user} do
      valid_params = %{
        "name" => "Oleh",
        "description" => "A cool author",
      }

      assert form =
        AshPhoenix.Form.for_create(Metadata.Author, :create,
          as: "author",
          actor: user
        )
        |> AshPhoenix.Form.ensure_can_submit!()
    end
  end

  describe "create new author via the form" do
    test "just name and description", %{user: user} do
      valid_params = %{
        "name" => "Oleh",
        "description" => "A cool author",
      }

      form =
        AshPhoenix.Form.for_create(Metadata.Author, :create,
          as: "author",
          actor: user
        )

      assert {:ok, author} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end

  describe "update existing author via the form" do
    test "update author's name", %{user: user} do
      author = generate(author())

      valid_params = %{
        "name" => "Another Oleh",
        "description" => "A cool author",
      }

      form =
        AshPhoenix.Form.for_update(author, :update,
          as: "author",
          actor: user
        )

      assert {:ok, author} = AshPhoenix.Form.submit(form, params: valid_params)
    end
  end
end
