defmodule DecentralisedBookIndex.Sync.DataTransformers.PublisherSyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.PublisherSync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "sync publisher transformations" do
    test "a new publisher will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      publisher = %{
        id: "11349865-1b7b-454a-b999-6c4059888a78",
        name: "Publisher",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      {:ok, inserted_at, 0} = DateTime.from_iso8601(publisher[:inserted_at])
      {:ok, updated_at, 0} = DateTime.from_iso8601(publisher[:updated_at])

      assert :ok = PublisherSync.create_update(publisher, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      publisher =
        publisher
        |> Map.replace(:inserted_at, inserted_at)
        |> Map.replace(:updated_at, updated_at)

      assert publisher = saved_publisher
      assert server.id == saved_publisher.dbi_server_id
    end

    test "update an existing publisher", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, publisher} = Metadata.create_publisher("Publisher", actor: user)

      publisher_attrs = %{
        id: publisher.id,
        name: "Publisher",
        inserted_at: "2025-03-21T09:20:48.791539Z",
        updated_at: "2025-03-21T09:20:48.791539Z"
      }

      assert :ok = PublisherSync.create_update(publisher_attrs, server.id)
      assert {:ok, saved_publisher} = Metadata.get_publisher_by_id(publisher.id)

      assert publisher_attrs = saved_publisher
      assert server.id == saved_publisher.dbi_server_id
    end
  end
end
